@@ -684,8 +684,8 @@ void mouseButtonDownHandler(uint8_t mouseButton)
 		return;
 	}
 
-	// mouse 0,0 = open exit dialog
-	if (mouse.x == 0 && mouse.y == 0)
+	// mouse 0,0 = open exit dialog (also make sure the test always works in fullscreen mode)
+	if ((mouse.x == 0 && mouse.y == 0) || (video.fullscreen && (video.renderX > 0 || video.renderY > 0) && (mouse.rawX == 0 && mouse.rawY == 0)))
 	{
 		if (quitBox(false) == 1)
 			editor.throwExit = true;
@@ -834,8 +834,12 @@ void readMouseXY(void)
 	{
 		mouse.buttonState = SDL_GetGlobalMouseState(&mx, &my);
 
+		mouse.absX = mx;
+		mouse.absY = my;
+
 		// convert desktop coords to window coords
 		SDL_GetWindowPosition(video.window, &windowX, &windowY);
+
 		mx -= windowX;
 		my -= windowY;
 	}
@@ -843,8 +847,14 @@ void readMouseXY(void)
 	{
 		// special mode for KMSDRM (XXX: Confirm that this still works...)
 		mouse.buttonState = SDL_GetMouseState(&mx, &my);
+
+		mouse.absX = mx;
+		mouse.absY = my;
 	}
 
+	mouse.rawX = mx;
+	mouse.rawY = my;
+
 	if (video.fullscreen)
 	{
 		// centered fullscreen mode (not stretched) needs further coord translation
