@@ -51,15 +51,16 @@ static char wndTitle[256];
 static sprite_t sprites[SPRITE_NUM];
 
 // for FPS counter
+#define FPS_LINES 15
 #define FPS_SCAN_FRAMES 60
 #define FPS_RENDER_W 285
-#define FPS_RENDER_H (((FONT1_CHAR_H + 1) * 8) + 1)
+#define FPS_RENDER_H (((FONT1_CHAR_H + 1) * FPS_LINES) + 1)
 #define FPS_RENDER_X 2
 #define FPS_RENDER_Y 2
 
 static char fpsTextBuf[1024];
 static uint64_t frameStartTime;
-static double dRunningFPS, dFrameTime, dAvgFPS;
+static double dRunningFrameDuration, dAvgFPS;
 // ------------------
 
 static void drawReplayerData(void);
@@ -68,8 +69,7 @@ void resetFPSCounter(void)
 {
 	editor.framesPassed = 0;
 	fpsTextBuf[0] = '\0';
-	dRunningFPS = VBLANK_HZ;
-	dFrameTime = 1000.0 / VBLANK_HZ;
+	dRunningFrameDuration = 1000.0 / VBLANK_HZ;
 }
 
 void beginFPSCounter(void)
@@ -80,13 +80,17 @@ void beginFPSCounter(void)
 
 static void drawFPSCounter(void)
 {
+	SDL_version SDLVer;
+
+	SDL_GetVersion(&SDLVer);
+
 	if (editor.framesPassed >= FPS_SCAN_FRAMES && (editor.framesPassed % FPS_SCAN_FRAMES) == 0)
 	{
-		dAvgFPS = dRunningFPS * (1.0 / FPS_SCAN_FRAMES);
+		dAvgFPS = 1000.0 / (dRunningFrameDuration / FPS_SCAN_FRAMES);
 		if (dAvgFPS < 0.0 || dAvgFPS > 99999999.9999)
 			dAvgFPS = 99999999.9999; // prevent number from overflowing text box
 
-		dRunningFPS = 0.0;
+		dRunningFrameDuration = 0.0;
 	}
 
 	clearRect(FPS_RENDER_X+2, FPS_RENDER_Y+2, FPS_RENDER_W, FPS_RENDER_H);
@@ -95,10 +99,12 @@ static void drawFPSCounter(void)
 	hLineDouble(FPS_RENDER_X+1, FPS_RENDER_Y, FPS_RENDER_W, PAL_FORGRND);
 	hLineDouble(FPS_RENDER_X+1, FPS_RENDER_Y+FPS_RENDER_H+2, FPS_RENDER_W, PAL_FORGRND);
 
-	// test if enough data is collected yet
+	// if enough frame data isn't collected yet, show a message
 	if (editor.framesPassed < FPS_SCAN_FRAMES)
 	{
-		textOut(FPS_RENDER_X+53, FPS_RENDER_Y+39, PAL_FORGRND, "Collecting frame information...");
+		const char *text = "Gathering frame information...";
+		const uint16_t textW = textWidth(text);
+		textOut(FPS_RENDER_X+((FPS_RENDER_W/2)-(textW/2)), FPS_RENDER_Y+((FPS_RENDER_H/2)-(FONT1_CHAR_H/2)), PAL_FORGRND, text);
 		return;
 	}
 
@@ -110,20 +116,36 @@ static void drawFPSCounter(void)
 	if (dAudLatency < 0.0 || dAudLatency > 999999999.9999)
 		dAudLatency = 999999999.9999; // prevent number from overflowing text box
 
-	sprintf(fpsTextBuf, "Frames per second: %.4f\n" \
+	sprintf(fpsTextBuf,
+	             "SDL version: %u.%u.%u\n" \
+	             "Frames per second: %.3f\n" \
 	             "Monitor refresh rate: %.1fHz (+/-)\n" \
 	             "59..61Hz GPU VSync used: %s\n" \
 	             "Audio frequency: %.1fkHz (expected %.1fkHz)\n" \
 	             "Audio buffer samples: %d (expected %d)\n" \
 	             "Audio channels: %d (expected %d)\n" \
 	             "Audio latency: %.1fms (expected %.1fms)\n" \
+	             "Render size: %dx%d (offset %d,%d)\n" \
+	             "Disp. size: %dx%d (window: %dx%d)\n" \
+	             "Render scaling: x=%.2f, y=%.2f\n" \
+	             "Mouse to render space muls: x=%.2f, y=%.2f\n" \
+	             "Relative mouse coords: %d,%d\n" \
+	             "Absolute mouse coords: %d,%d\n" \
 	             "Press CTRL+SHIFT+F to close this box.\n",
-	             dAvgFPS, dRefreshRate,
+	             SDLVer.major, SDLVer.minor, SDLVer.patch,
+	             dAvgFPS,
+	             dRefreshRate,
 	             video.vsync60HzPresent ? "yes" : "no",
 	             audio.haveFreq * (1.0 / 1000.0), audio.wantFreq * (1.0 / 1000.0),
 	             audio.haveSamples, audio.wantSamples,
 	             audio.haveChannels, audio.wantChannels,
-	             dAudLatency, ((audio.wantSamples * 1000.0) / audio.wantFreq));
+	             dAudLatency, ((audio.wantSamples * 1000.0) / audio.wantFreq),
+	             video.renderW, video.renderH, video.renderX, video.renderY,
+	             video.displayW, video.displayH, video.windowW, video.windowH,
+	             video.renderW / (float)SCREEN_W, video.renderH / (float)SCREEN_H,
+	             video.fMouseXMul, video.fMouseYMul,
+	             mouse.x, mouse.y,
+	             mouse.absX, mouse.absY);
 
 	// draw text
 
@@ -137,22 +159,35 @@ static void drawFPSCounter(void)
 		if (ch == '\n')
 		{
 			yPos += FONT1_CHAR_H+1;
-			xPos = FPS_RENDER_X + 3;
+			xPos = FPS_RENDER_X+3;
 			continue;
 		}
 
 		charOut(xPos, yPos, PAL_FORGRND, ch);
 		xPos += charWidth(ch);
 	}
+
+	// draw framerate tester symbol
+
+	const uint16_t symbolEnd = 115;
+
+	// ping-pong movement
+	uint16_t x = editor.framesPassed % (symbolEnd * 2);
+	if (x >= symbolEnd)
+		x = (symbolEnd * 2) - x;
+
+	charOut(164 + x, 16, PAL_FORGRND, '*');
 }
 
 void endFPSCounter(void)
 {
 	if (video.showFPSCounter && frameStartTime > 0)
 	{
-		const uint64_t frameTimeDiff = SDL_GetPerformanceCounter() - frameStartTime;
-		const double dHz = 1000.0 / (frameTimeDiff * editor.dPerfFreqMulMs);
-		dRunningFPS += dHz;
+		uint64_t frameTimeDiff64 = SDL_GetPerformanceCounter() - frameStartTime;
+		if (frameTimeDiff64 > INT32_MAX)
+			frameTimeDiff64 = INT32_MAX;
+
+		dRunningFrameDuration += (int32_t)frameTimeDiff64 / (hpcFreq.dFreq / 1000.0);
 	}
 }
 
@@ -238,6 +273,8 @@ static void updateRenderSizeVars(void)
 		SDL_GetWindowSize(video.window, &video.displayW, &video.displayH);
 	}
 
+	SDL_GetWindowSize(video.window, &video.windowW, &video.windowH);
+
 	if (video.fullscreen)
 	{
 		if (config.specialFlags2 & STRETCH_IMAGE)
@@ -271,8 +308,8 @@ static void updateRenderSizeVars(void)
 			if (dXUpscale != 0.0) video.renderW = (int32_t)(video.renderW / dXUpscale);
 			if (dYUpscale != 0.0) video.renderH = (int32_t)(video.renderH / dYUpscale);
 
-			video.renderX = (video.displayW - video.renderW) >> 1;
-			video.renderY = (video.displayH - video.renderH) >> 1;
+			video.renderX = (video.displayW - video.renderW) / 2;
+			video.renderY = (video.displayH - video.renderH) / 2;
 		}
 	}
 	else
@@ -284,19 +321,26 @@ static void updateRenderSizeVars(void)
 	}
 
 	// for mouse cursor creation
-	video.xScale = (uint32_t)round(video.renderW * (1.0 / SCREEN_W));
-	video.yScale = (uint32_t)round(video.renderH * (1.0 / SCREEN_H));
+	video.xScale = (uint32_t)round(video.renderW / (double)SCREEN_W);
+	video.yScale = (uint32_t)round(video.renderH / (double)SCREEN_H);
 
 	createMouseCursors();
 }
 
 void enterFullscreen(void)
 {
-	SDL_DisplayMode dm;
+	SDL_SetWindowFullscreen(video.window, SDL_WINDOW_FULLSCREEN_DESKTOP);
 
 	if (config.specialFlags2 & STRETCH_IMAGE)
 	{
-		SDL_GetDesktopDisplayMode(0, &dm);
+		SDL_DisplayMode dm;
+
+		int32_t di = SDL_GetWindowDisplayIndex(video.window);
+		if (di < 0)
+			di = 0; // return display index 0 (default) on error
+
+		SDL_GetDesktopDisplayMode(di, &dm);
+
 		SDL_RenderSetLogicalSize(video.renderer, dm.w, dm.h);
 	}
 	else
@@ -305,15 +349,17 @@ void enterFullscreen(void)
 	}
 
 	SDL_SetWindowSize(video.window, SCREEN_W, SCREEN_H);
-	SDL_SetWindowFullscreen(video.window, SDL_WINDOW_FULLSCREEN_DESKTOP);
+
+#ifndef __unix__ // can be severely buggy on Linux... (at least when used like this)
 	SDL_SetWindowGrab(video.window, SDL_TRUE);
+#endif
 
 	updateRenderSizeVars();
 	updateMouseScaling();
 	setMousePosToCenter();
 }
 
-void leaveFullScreen(void)
+void leaveFullscreen(void)
 {
 	SDL_SetWindowFullscreen(video.window, 0);
 	SDL_RenderSetLogicalSize(video.renderer, SCREEN_W, SCREEN_H);
@@ -321,21 +367,28 @@ void leaveFullScreen(void)
 	setWindowSizeFromConfig(false);
 
 	SDL_SetWindowSize(video.window, SCREEN_W * video.upscaleFactor, SCREEN_H * video.upscaleFactor);
+
+#ifndef __unix__
 	SDL_SetWindowGrab(video.window, SDL_FALSE);
+#endif
 
 	updateRenderSizeVars();
 	updateMouseScaling();
 	setMousePosToCenter();
+
+#ifdef __unix__ // can be required on Linux... (or else the window keeps moving down every time you leave fullscreen)
+	SDL_SetWindowPosition(video.window, SDL_WINDOWPOS_CENTERED, SDL_WINDOWPOS_CENTERED);
+#endif
 }
 
-void toggleFullScreen(void)
+void toggleFullscreen(void)
 {
 	video.fullscreen ^= 1;
 
 	if (video.fullscreen)
 		enterFullscreen();
 	else
-		leaveFullScreen();
+		leaveFullscreen();
 }
 
 bool setupSprites(void)
@@ -742,13 +795,11 @@ void closeVideo(void)
 		video.window = NULL;
 	}
 
-	if (video.frameBufferUnaligned != NULL)
+	if (video.frameBuffer != NULL)
 	{
-		free(video.frameBufferUnaligned);
-		video.frameBufferUnaligned = NULL;
+		free(video.frameBuffer);
+		video.frameBuffer = NULL;
 	}
-
-	video.frameBuffer = NULL;
 }
 
 void setWindowSizeFromConfig(bool updateRenderer)
@@ -773,8 +824,12 @@ void setWindowSizeFromConfig(bool updateRenderer)
 	uint8_t oldUpscaleFactor = video.upscaleFactor;
 	if (config.windowFlags & WINSIZE_AUTO)
 	{
+		int32_t di = SDL_GetWindowDisplayIndex(video.window);
+		if (di < 0)
+			di = 0; // return display index 0 (default) on error
+
 		// find out which upscaling factor is the biggest to fit on screen
-		if (SDL_GetDesktopDisplayMode(0, &dm) == 0)
+		if (SDL_GetDesktopDisplayMode(di, &dm) == 0)
 		{
 			for (i = MAX_UPSCALE_FACTOR; i >= 1; i--)
 			{
@@ -831,17 +886,31 @@ void updateWindowTitle(bool forceUpdate)
 		strncpy(songTitleTrunc, songTitle, sizeof (songTitleTrunc)-1);
 		songTitleTrunc[sizeof (songTitleTrunc)-1] = '\0';
 
-		if (song.isModified)
-			sprintf(wndTitle, "Fasttracker II clone v%s (%d-bit) - \"%s\" (unsaved)", PROG_VER_STR, CPU_BITS, songTitleTrunc);
-		else
-			sprintf(wndTitle, "Fasttracker II clone v%s (%d-bit) - \"%s\"", PROG_VER_STR, CPU_BITS, songTitleTrunc);
+#if CPU_BITS==32
+			if (song.isModified)
+				sprintf(wndTitle, "Fasttracker II clone v%s (32-bit) - \"%s\" (unsaved)", PROG_VER_STR, songTitleTrunc);
+			else
+				sprintf(wndTitle, "Fasttracker II clone v%s (32-bit) - \"%s\"", PROG_VER_STR, songTitleTrunc);
+#else
+			if (song.isModified)
+				sprintf(wndTitle, "Fasttracker II clone v%s - \"%s\" (unsaved)", PROG_VER_STR, songTitleTrunc);
+			else
+				sprintf(wndTitle, "Fasttracker II clone v%s - \"%s\"", PROG_VER_STR, songTitleTrunc);
+#endif
 	}
 	else
 	{
+#if CPU_BITS==32
 		if (song.isModified)
-			sprintf(wndTitle, "Fasttracker II clone v%s (%d-bit) - \"untitled\" (unsaved)", PROG_VER_STR, CPU_BITS);
+			sprintf(wndTitle, "Fasttracker II clone v%s (32-bit) - \"untitled\" (unsaved)", PROG_VER_STR);
 		else
-			sprintf(wndTitle, "Fasttracker II clone v%s (%d-bit) - \"untitled\"", PROG_VER_STR, CPU_BITS);
+			sprintf(wndTitle, "Fasttracker II clone v%s (32-bit) - \"untitled\"", PROG_VER_STR);
+#else
+		if (song.isModified)
+			sprintf(wndTitle, "Fasttracker II clone v%s - \"untitled\" (unsaved)", PROG_VER_STR);
+		else
+			sprintf(wndTitle, "Fasttracker II clone v%s - \"untitled\"", PROG_VER_STR);
+#endif
 	}
 
 	SDL_SetWindowTitle(video.window, wndTitle);
@@ -885,7 +954,11 @@ bool setupWindow(void)
 
 	setWindowSizeFromConfig(false);
 
-	SDL_GetDesktopDisplayMode(0, &dm);
+	int32_t di = SDL_GetWindowDisplayIndex(video.window);
+	if (di < 0)
+		di = 0; // return display index 0 (default) on error
+
+	SDL_GetDesktopDisplayMode(di, &dm);
 	video.dMonitorRefreshRate = (double)dm.refresh_rate;
 
 	if (dm.refresh_rate >= 59 && dm.refresh_rate <= 61)
@@ -941,7 +1014,7 @@ bool setupRenderer(void)
 
 	SDL_RenderSetLogicalSize(video.renderer, SCREEN_W, SCREEN_H);
 
-#if SDL_PATCHLEVEL >= 5
+#if SDL_MINOR_VERSION >= 24 || (SDL_MINOR_VERSION == 0 && SDL_PATCHLEVEL >= 5)
 	SDL_RenderSetIntegerScale(video.renderer, SDL_TRUE);
 #endif
 
@@ -955,19 +1028,22 @@ bool setupRenderer(void)
 	}
 
 	// framebuffer used by SDL (for texture)
-	video.frameBufferUnaligned = (uint32_t *)MALLOC_PAD(SCREEN_W * SCREEN_H * sizeof (int32_t), 256);
-	if (video.frameBufferUnaligned == NULL)
+	video.frameBuffer = (uint32_t *)malloc(SCREEN_W * SCREEN_H * sizeof (int32_t));
+	if (video.frameBuffer == NULL)
 	{
 		showErrorMsgBox("Not enough memory!");
 		return false;
 	}
 
-	// we want an aligned pointer
-	video.frameBuffer = (uint32_t *)ALIGN_PTR(video.frameBufferUnaligned, 256);
-
 	if (!setupSprites())
 		return false;
 
+	// Workaround: SDL_GetGlobalMouseState() doesn't work with KMSDRM/Wayland
+	video.useDesktopMouseCoords = true;
+	const char *videoDriver = SDL_GetCurrentVideoDriver();
+	if (videoDriver != NULL && (strcmp("KMSDRM", videoDriver) == 0 || strcmp("wayland", videoDriver) == 0))
+		video.useDesktopMouseCoords = false;
+
 	updateRenderSizeVars();
 	updateMouseScaling();
 
@@ -976,12 +1052,6 @@ bool setupRenderer(void)
 	else
 		SDL_ShowCursor(SDL_FALSE);
 
-	// Workaround: SDL_GetGlobalMouseState() doesn't work with KMSDRM/Wayland
-	video.useDesktopMouseCoords = true;
-	const char *videoDriver = SDL_GetCurrentVideoDriver();
-	if (videoDriver != NULL && (strcmp("KMSDRM", videoDriver) == 0 || strcmp("wayland", videoDriver) == 0))
-		video.useDesktopMouseCoords = false;
-
 	SDL_SetRenderDrawColor(video.renderer, 0, 0, 0, SDL_ALPHA_OPAQUE);
 	return true;
 }
